import sys, networkx, logging
from qbindiff import GraphDiffer, Program
from qbindiff.types import SimMatrix
from typing import Any


def load_graph(filename: str) -> networkx.Graph:
    """Load the undirected PPI graph"""
    with open(filename) as f:
        lines = f.readlines()

    graph = networkx.Graph()

    for line in lines:
        label1, label2 = line.strip().split("\t")
        graph.add_edge(label1, label2)

    return graph


def load_blast(
    filename: str, primary_nodes: set[str], secondary_nodes: set[str]
) -> dict[tuple[str, str], float]:
    """
    Load the BLAST scores for each edge (node1, node2).
    It returns a dictionary { (node1, node2) -> score }
    """
    with open(filename) as f:
        lines = f.readlines()

    scores = {}
    for line in lines:
        label1, label2, score = line.strip().split("\t")
        if label1 in primary_nodes and label2 in secondary_nodes:
            scores[(label1, label2)] = float(score)
        elif label1 in secondary_nodes and label2 in primary_nodes:
            scores[(label2, label1)] = float(score)
        else:
            # Ignoring score
            pass

    return scores


def init_similarity_matrix(
    sim_matrix: SimMatrix,
    primary: Program,
    secondary: Program,
    primary_mapping: dict[Any, int],
    secondary_mapping: dict[Any, int],
    blast: dict[tuple[str, str], float],
) -> None:
    sim_matrix[:] = 0
    for (label1, label2), score in blast.items():
        sim_matrix[primary_mapping[label1], secondary_mapping[label2]] = score

    # Normalize
    sim_matrix /= sim_matrix.max()


logger = logging.getLogger()
logger.setLevel(logging.DEBUG)


if len(sys.argv) != 4:
    print(f"Usage ./{sys.argv[0]} GRAPH1 GRAPH2 BLAST_SCORE")
    sys.exit(1)

primary: networkx.Graph = load_graph(sys.argv[1])
print(f"Primary graph\n\tNodes: {len(primary)}  Edges: {primary.size()}")
secondary: networkx.Graph = load_graph(sys.argv[2])
print(f"Secondary graph\n\tNodes: {len(secondary)}  Edges: {secondary.size()}")

blast_scores: dict[tuple[str, str], float] = load_blast(sys.argv[3], set(primary.nodes), set(secondary.nodes))
print(f"Size of similarity matrix {len(blast)}")

differ = GraphDiffer(primary, secondary, sparsity_ratio=0)
differ.register_prepass(init_similarity_matrix, blast=blast_scores)

mapping = differ.compute_matching()
print(
    f"Similarity: {mapping.similarity}  Normalized Similarity: {mapping.normalized_similarity}"
)

